"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const errors_1 = require("request-promise-native/errors");
const helper_1 = __importDefault(require("../utils/helper"));
const problem_1 = __importDefault(require("./problem"));
class Leetcode {
    session;
    csrfToken;
    static uris;
    static setUris(uris) {
        Leetcode.uris = uris;
    }
    constructor(credit) {
        this.session = credit.session;
        this.csrfToken = credit.csrfToken;
    }
    get credit() {
        return {
            session: this.session,
            csrfToken: this.csrfToken,
        };
    }
    static async build(username, password, endpoint) {
        helper_1.default.switchEndPoint(endpoint);
        const credit = await this.login(username, password);
        helper_1.default.setCredit(credit);
        return new Leetcode(credit);
    }
    static async login(username, password) {
        // got login token first
        const response = await helper_1.default.HttpRequest({
            url: Leetcode.uris.login,
            resolveWithFullResponse: true,
        });
        const token = helper_1.default.parseCookie(response.headers["set-cookie"], "csrftoken");
        // Leetcode CN return null here, but it's does not matter
        let credit = {
            csrfToken: token,
        };
        helper_1.default.setCredit(credit);
        try {
            // then login
            const _response = await helper_1.default.HttpRequest({
                method: "POST",
                url: Leetcode.uris.login,
                form: {
                    csrfmiddlewaretoken: token,
                    login: username,
                    password: password,
                },
                resolveWithFullResponse: true,
            });
            const session = helper_1.default.parseCookie(_response.headers["set-cookie"], "LEETCODE_SESSION");
            const csrfToken = helper_1.default.parseCookie(_response.headers["set-cookie"], "csrftoken");
            credit = {
                session: session,
                csrfToken: csrfToken,
            };
        }
        catch (e) {
            if (e instanceof errors_1.StatusCodeError) {
                throw new Error("Login Fail");
            }
        }
        return credit;
    }
    async getProfile() {
        // ? TODO : fetch more user profile.
        const response = await helper_1.default.GraphQLRequest({
            query: `
            {
                user {
                    username
                }
            }
            `,
        });
        return response.user;
    }
    async getAllProblems() {
        let response = await helper_1.default.HttpRequest({
            url: Leetcode.uris.problemsAll,
        });
        response = JSON.parse(response);
        const problems = response.stat_status_pairs.map((p) => {
            return new problem_1.default(p.stat.question__title_slug, p.stat.question_id, p.stat.question__title, helper_1.default.difficultyMap(p.difficulty.level), p.is_favor, p.paid_only, undefined, undefined, helper_1.default.statusMap(p.status), undefined, p.stat.total_acs, p.stat.total_submitted, undefined, undefined, undefined);
        });
        return problems;
    }
    async getProblemsByTag(tag) {
        const response = await helper_1.default.GraphQLRequest({
            query: `
                query getTopicTag($slug: String!) {
                    topicTag(slug: $slug) {
                        questions {
                            status
                            questionId
                            title
                            titleSlug
                            stats
                            difficulty
                            isPaidOnly
                            topicTags {
                                slug
                            }
                        }
                    }
                }
            `,
            variables: {
                slug: tag,
            },
        });
        const problems = response.topicTag.questions.map((p) => {
            const stat = JSON.parse(p.stats);
            return new problem_1.default(p.titleSlug, p.questionId, p.title, stat.title, undefined, p.isPaidOnly, undefined, undefined, helper_1.default.statusMap(p.status), p.topicTags.map((t) => t.slug), stat.totalAcceptedRaw, stat.totalSubmissionRaw, undefined, undefined, undefined);
        });
        return problems;
    }
}
exports.default = Leetcode;
//# sourceMappingURL=leetcode.js.map