"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const helper_1 = __importDefault(require("../utils/helper"));
const interfaces_1 = require("../utils/interfaces");
const submission_1 = __importDefault(require("./submission"));
class Problem {
    slug;
    id;
    title;
    difficulty;
    starred;
    locked;
    likes;
    dislikes;
    status;
    tag;
    totalAccepted;
    totalSubmission;
    sampleTestCase;
    content;
    translatedContent;
    codeSnippets;
    static uris;
    static setUris(uris) {
        Problem.uris = uris;
    }
    constructor(slug, id, title, difficulty, starred, locked, likes, dislikes, status, tag, totalAccepted, totalSubmission, sampleTestCase, content, translatedContent, codeSnippets) {
        this.slug = slug;
        this.id = id;
        this.title = title;
        this.difficulty = difficulty;
        this.starred = starred;
        this.locked = locked;
        this.likes = likes;
        this.dislikes = dislikes;
        this.status = status;
        this.tag = tag;
        this.totalAccepted = totalAccepted;
        this.totalSubmission = totalSubmission;
        this.sampleTestCase = sampleTestCase;
        this.content = content;
        this.translatedContent = translatedContent;
        this.codeSnippets = codeSnippets;
    }
    async detail() {
        const response = await helper_1.default.GraphQLRequest({
            query: `
                query getQuestionDetail($titleSlug: String!) {
                    question(titleSlug: $titleSlug) {
                        questionId
                        title
                        difficulty
                        likes
                        dislikes
                        isLiked
                        isPaidOnly
                        stats
                        status
                        content
                        ${helper_1.default.endpoint === interfaces_1.EndPoint.CN
                ? "translatedContent"
                : ""}
                        topicTags {
                            name
                        }
                        codeSnippets {
                            lang
                            langSlug
                            code
                        }
                        sampleTestCase
                    }
                }
            `,
            variables: {
                titleSlug: this.slug,
            },
        });
        const question = response.question;
        this.id = Number(question.questionId);
        this.title = question.title;
        this.difficulty = helper_1.default.difficultyMap(question.difficulty);
        this.starred = question.isLiked !== null;
        this.locked = question.isPaidOnly;
        this.likes = question.likes;
        this.dislikes = question.dislikes;
        this.status = helper_1.default.statusMap(question.status);
        this.tag = question.topicTags.map(function (t) {
            return t.name;
        });
        const stats = JSON.parse(question.stats);
        this.totalAccepted = stats.totalAcceptedRaw;
        this.totalSubmission = stats.totalSubmissionRaw;
        this.sampleTestCase = question.sampleTestCase;
        this.content = question.content;
        this.translatedContent = question.translatedContent;
        this.codeSnippets = question.codeSnippets;
        return this;
    }
    async getSubmissions() {
        const submissions = [];
        let offset = 0;
        const limit = 20;
        let hasNext = true;
        while (hasNext) {
            const response = await helper_1.default.GraphQLRequest({
                query: `
                query Submissions($offset: Int!, $limit: Int!, $questionSlug: String!) {
                    submissionList(offset: $offset, limit: $limit, questionSlug: $questionSlug) {
                        lastKey
                        hasNext
                        submissions {
                            id
                            statusDisplay
                            lang
                            runtime
                            timestamp
                            url
                            isPending
                            memory
                        }
                    }
                }
                `,
                variables: {
                    offset: offset,
                    limit: limit,
                    questionSlug: this.slug,
                },
            });
            hasNext = response.submissionList.hasNext;
            const submission = response.submissionList.submissions;
            offset += submission.length;
            submission.map((s) => {
                submissions.push(new submission_1.default(Number(s.id), s.lang, s.memory, s.runtime, s.statusDisplay, Number(s.timestamp)));
            });
        }
        return submissions;
    }
    async submit(lang, code) {
        const response = await helper_1.default.HttpRequest({
            url: Problem.uris.submit.replace("$slug", this.slug),
            method: "POST",
            body: {
                lang: lang,
                // eslint-disable-next-line @typescript-eslint/camelcase
                question_id: this.id,
                // eslint-disable-next-line @typescript-eslint/camelcase
                typed_code: code,
            },
        });
        return new submission_1.default(JSON.parse(response).submission_id);
    }
}
exports.default = Problem;
//# sourceMappingURL=problem.js.map