"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const graphql_request_1 = require("graphql-request");
const request_promise_native_1 = __importDefault(require("request-promise-native"));
const config_1 = __importDefault(require("../lib/config"));
const contest_1 = __importDefault(require("../lib/contest"));
const leetcode_1 = __importDefault(require("../lib/leetcode"));
const problem_1 = __importDefault(require("../lib/problem"));
const submission_1 = __importDefault(require("../lib/submission"));
const interfaces_1 = require("./interfaces");
class Helper {
    static credit;
    static uris;
    static endpoint;
    static setCredit(credit) {
        Helper.credit = credit;
    }
    static setUris(uris) {
        Helper.uris = uris;
    }
    static setEndpoint(endpoint) {
        Helper.endpoint = endpoint;
    }
    static parseCookie(cookies, key) {
        if (!cookies) {
            return "";
        }
        for (let ix = 0; ix !== cookies.length; ++ix) {
            const result = cookies[ix].match(new RegExp(`${key}=(.+?);`));
            if (result) {
                return result[1] || "";
            }
        }
        return "";
    }
    static levelToName(level) {
        switch (level) {
            case 1:
                return "Easy";
            case 2:
                return "Medium";
            case 3:
                return "Hard";
            default:
                return "";
        }
    }
    static statusMap(status) {
        switch (status) {
            case "ac":
                return interfaces_1.ProblemStatus["Accept"];
            case "notac":
                return interfaces_1.ProblemStatus["Not Accept"];
            case null:
                return interfaces_1.ProblemStatus["Not Start"];
            default:
                return interfaces_1.ProblemStatus["Not Start"];
        }
    }
    static difficultyMap(difficulty) {
        switch (difficulty) {
            case 1:
                return interfaces_1.ProblemDifficulty.Easy;
            case 2:
                return interfaces_1.ProblemDifficulty.Medium;
            case 3:
                return interfaces_1.ProblemDifficulty.Hard;
            default:
                return interfaces_1.ProblemDifficulty.Easy;
        }
    }
    static submissionStatusMap(submission) {
        switch (submission) {
            case "Accepted":
                return interfaces_1.SubmissionStatus["Accepted"];
            case "Compile Error":
                return interfaces_1.SubmissionStatus["Compile Error"];
            case "Time Limit Exceeded":
                return interfaces_1.SubmissionStatus["Time Limit Exceeded"];
            case "Wrong Answer":
                return interfaces_1.SubmissionStatus["Wrong Answer"];
            case "10":
                return interfaces_1.SubmissionStatus["Accepted"];
            case "11":
                return interfaces_1.SubmissionStatus["Wrong Answer"];
            case "14":
                return interfaces_1.SubmissionStatus["Time Limit Exceeded"];
            case "20":
                return interfaces_1.SubmissionStatus["Compile Error"];
            // TODO : find out what this numbers mean
            // 12 => MLE
            // 13 => OLE
            // 15 => RE
            // 16 => IE
            // 21 => UE
            // 30 => TO
            // default => UE
            default:
                return interfaces_1.SubmissionStatus["Wrong Answer"];
        }
    }
    static async HttpRequest(options) {
        return await (0, request_promise_native_1.default)({
            method: options.method || "GET",
            uri: options.url,
            followRedirect: false,
            headers: {
                Cookie: Helper.credit
                    ? `LEETCODE_SESSION=${Helper.credit.session};csrftoken=${Helper.credit.csrfToken}`
                    : "",
                "X-Requested-With": "XMLHttpRequest",
                "X-CSRFToken": Helper.credit ? Helper.credit.csrfToken : "",
                Referer: options.referer || Helper.uris.base,
            },
            resolveWithFullResponse: options.resolveWithFullResponse || false,
            form: options.form || null,
            body: JSON.stringify(options.body) || "",
        });
    }
    static async GraphQLRequest(options) {
        const client = new graphql_request_1.GraphQLClient(Helper.uris.graphql, {
            headers: {
                Origin: options.origin || Helper.uris.base,
                Referer: options.referer || Helper.uris.base,
                Cookie: `LEETCODE_SESSION=${Helper.credit.session};csrftoken=${Helper.credit.csrfToken};`,
                "X-Requested-With": "XMLHttpRequest",
                "X-CSRFToken": Helper.credit.csrfToken,
            },
        });
        return await client.request(options.query, options.variables || {});
    }
    static switchEndPoint(endPoint) {
        Helper.setEndpoint(endPoint);
        const uris = (() => {
            switch (endPoint) {
                case interfaces_1.EndPoint.US:
                    return config_1.default.uri.us;
                case interfaces_1.EndPoint.CN:
                    return config_1.default.uri.cn;
            }
        })();
        Helper.setUris(uris);
        leetcode_1.default.setUris(uris);
        problem_1.default.setUris(uris);
        contest_1.default.setUris(uris);
        submission_1.default.setUris(uris);
    }
}
exports.default = Helper;
//# sourceMappingURL=helper.js.map